/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright (C) Yeico S. A. de C. V.
 * xlsLib -- A multiplatform, C++ library for dynamic generation of Excel (TM) 
 * files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  
 * $Source: /cvsroot/xlslib/xlslib/src/xlslib/workbook.h,v $
 * $Revision: 1.2 $
 * $Author: darioglz $
 * $Date: 2004/09/01 00:47:21 $
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 * File description:
 *
 *
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef WORKBOOK_H
#define WORKBOOK_H 


#include <config.h>
#include "common.h"
#include "oledoc.h"
#include "datast.h"
#include "globalrec.h"
#include "sheetrec.h"
#include "recdef.h"
#include "continue.h"
#include "summinfo.h"
#include "docsumminfo.h"
#include <sstream>

namespace xlslib_core
{

  typedef enum
    {
      WB_INIT,
      WB_GLOBALRECORDS,
      WB_SHEETS,
      WB_CONTINUE_REC,
      WB_FINISH
    } WorkbookDumpState_t;



  typedef enum
    {
      STK_INLINE=0,
      STK_ATTACHMENT

    }Stroke_Disposition_t;

  class workbook
    :public COleDoc
    {

    private:

      CSummaryInfo m_SummaryInfo;
      CDocSummaryInfo m_DocSummaryInfo;

      CGlobalRecords m_GlobalRecords;
      Sheets_Vector_t m_Sheets;
      WorkbookDumpState_t m_DumpState;
      WorkbookDumpState_t m_PreviousDumpState;

      CUnit* m_pCurrentData;


      // Continue record variables:
      CUnit* m_pContinueRecord;
      unsigned16_t m_ContinueIndex;

    protected:

    public:
      workbook();
      workbook(write_fptr_t w_fptr);
      ~workbook();

      worksheet* sheet(std::string sheetname);
      worksheet* GetSheet(unsigned16_t sheetnum);
   
      font_t* font(std::string   name);
      format_t* format(std::string  formatstr);


      xf_t* xformat(void);
      xf_t* xformat(font_t* font);

      CUnit* DumpData(void);

      void Dump(std::string filename);

      void Disposition(Stroke_Disposition_t disp);

      void Stroke(std::string filename);

      Stroke_Disposition_t m_Disposition;

    private:

      bool         dump_not_started;
      unsigned32_t globalrec_size;
      unsigned16_t current_sheet;
      unsigned32_t offset;               



    };

}
#endif //WORKBOOK_H

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * $Log: workbook.h,v $
 * Revision 1.2  2004/09/01 00:47:21  darioglz
 * + Modified to gain independence of target
 *
 * Revision 1.1.1.1  2004/08/27 16:31:44  darioglz
 * Initial Import.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


